<h2>Press</h2>

<h3>Web Articles on Sequel</h3>

<ul>
<li><a href='http://rubydev.ru/2011/10/interview-with-jeremy-evans-about-sequel-and-not-only/'>rubydev.ru: Interview with Jeremy Evans about Sequel and not only (October 2011)</a> (<a href="http://rubydev.ru/2011/10/interview_jeremy_evans_about_sequel_ru/">Russian Translation</a>)</li>
<li><a href='http://on-ruby.blogspot.com/2009/01/sequel-interview-with-jeremy-evans.html'>On Ruby: Sequel Interview with Jeremy Evans (January 2009)</a></li>
<li><a href='http://www.infoq.com/news/2008/11/sequel-ruby-db-toolkit'>InfoQ: Sequel Interview with Jeremy Evans (November 2008)</a></li>
<li><a href='http://on-ruby.blogspot.com/2008/01/sequel-interview-with-sharon-rosner.html'>On Ruby: Sequel Interview with Sharon Rosner (January 2008)</a></li>
<li><a href='http://www.oreillynet.com/ruby/blog/2007/07/ruby_project_spotlight_june_07.html'>O'Reilly Ruby: Ruby Project Spotlight: Sequel (July 2007)</a></li>
</ul>

<h3>Magazine Articles on Sequel</h3>

<ul>
<li><a href='http://www.impressjapan.jp/books/2579'>Ruby! Vol. 1</a> in japanese (thanks Shunichi Arai)</li>
</ul>

<h3><a href="http://groups.google.com/group/sequel-talk/browse_thread/thread/50726754599cff19">Companies and Individuals Using Sequel</a><h3>

<h2>Button</h2>

<p>You can use this button if you want to show that your site is powered by Sequel: <img src="images/sequel-button.png" /></p>
