<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">

<html xmlns="http://www.w3.org/1999/xhtml">
<head>
  <title>Sequel: The Database Toolkit for Ruby<%= " - #{title.capitalize}" unless title == 'index' %></title>

  <meta http-equiv="Content-Script-Type" content="text/javascript" />
  <meta http-equiv="Content-Style-Type" content="text/css" />
  <meta content="text/html; charset=utf-8" http-equiv="content-type" />
  <meta content="en" name="content-language" />

  <meta name="keywords" content="ruby, sequel, sql, toolkit, orm, plugins, activerecord, documentation, howto, news" />
  <meta name="description" content="Ruby Sequel is a lightweight database toolkit for Ruby." />

  <link href="css/application.css" media="screen" rel="stylesheet" type="text/css" />
  <link href="css/ruby.css" media="screen" rel="stylesheet" type="text/css" />
</head>
<body>
  <div id="header">
    <a href="index.html" id="logo"><img src="images/ruby-sequel.png" alt="Ruby Sequel Logo" /></a>
    <h1>The Database Toolkit for Ruby</h1>
    <div id="navigation">
      <ul>
        <li><a href="development.html">Development</a></li>
        <li><a href="documentation.html">Documentation</a></li>
        <li><a href="plugins.html">Plugins</a></li>
        <li><a href="press.html">Press</a></li>
        <li><a href="blog.html">Blog</a></li>
      </ul>
      <form action="http://www.google.com/search">
        <input type="hidden" name="sitesearch" value="sequel.jeremyevans.net" />
        <input id="searchbox" type="search" placeholder="Site Search" name="q" value="" />
      </form>
    </div>
    <br style="clear: both;" />
  </div>

  <hr />

  <div id="content">
    <br style="clear: both;" />
    <%= content %>
    <br style="clear: both;" />
  </div>
</body>
</html>
